<?
/*
*******************************
 IP-SYMCON Event Scripting
*******************************
File     : LCD_Functions.ips.php
Trigger  : 
Interval :
Author   : HJH
Project  : Utilities
Date     : 26.08.06
*/

//==============================================================================
//============================== Include File ==================================
//==============================================================================

//******************************************************************************
// Class OneWireLCD
//******************************************************************************
class OneWireLCD
{
 private static $Standard = "\x33\x32\x28\x0c\x01\x06";
 private static $ST7036   = "\x33\x32\x29\x1c\x52\x69\x74\x0c\x01\x06";
 private static $sol = array(0, 64, 20, 84);
 private static $ctab1 = array(''=>0x01, ''=>0x03, ''=>0x05,
                               ''=>0x02, ''=>0x04, ''=>0x06, ''=>0x07);
 private static $ctab2 = array(''=>0x84, ''=>0x94, ''=>0x81,
                               ''=>0x8e, ''=>0x99, ''=>0x9a, ''=>0x07);
 private static $asumlt = "\x0a\x00\x0e\x01\x0f\x11\x0f\x00";
 private static $acumlt = "\x11\x04\x0a\x11\x1f\x11\x11\x00";
 private static $osumlt = "\x0a\x00\x0e\x11\x11\x11\x0e\x00";
 private static $ocumlt = "\x11\x0e\x11\x11\x11\x11\x0e\x00";
 private static $usumlt = "\x0a\x00\x11\x11\x11\x13\x0d\x00";
 private static $ucumlt = "\x11\x00\x11\x11\x11\x11\x0e\x00";
 private static $ssharp = "\x06\x09\x11\x12\x11\x11\x16\x10";
 private $Controller;
 private $InitString;
 private $InstanceID;
 private $rows;
 private $cols;

 // public functions ===========================================================
 
 // Constructor ----------------------------------------------------------------
 public function __construct($InstanceID, $ControllerType, $rows, $cols)
 {
  $this->InstanceID = $InstanceID;
  $this->rows = $rows;
  $this->cols = $cols;
  $this->Controller = $ControllerType;
  switch ($this->Controller)
  {
   case "ST7036":
    $this->InitString = self::$ST7036;
    break;
   case "Standard":
   default:
    $this->InitString = self::$Standard;
    break;
  }
 }



 // LCD-Initialisierung --------------------------------------------------------
 public function SendLCDInit()
 {
  TMEX_F29_SetStrobe($this->InstanceID, TRUE);
  IPS_Sleep(100);
  $sendbuf = $this->MakeCmd($this->InitString);
  // Init-String ausgeben
  $this->WriteLCDData($sendbuf);
  // deutsche Sonderzeichen definieren
  if ($this->Controller == "ST7036")
  {
   $this->SendLCDCharDef(7, self::$ssharp);
  }
  else
  {
   $this->SendLCDCharDef(1, self::$asumlt);  // 
   $this->SendLCDCharDef(2, self::$acumlt);  // 
   $this->SendLCDCharDef(3, self::$osumlt);  // 
   $this->SendLCDCharDef(4, self::$ocumlt);  // 
   $this->SendLCDCharDef(5, self::$usumlt);  // 
   $this->SendLCDCharDef(6, self::$ucumlt);  // 
   $this->SendLCDCharDef(7, self::$ssharp);  // 
  }
 }



 // Display lschen ------------------------------------------------------------
 public function SendLCDClear()
 {
  $sendbuf = $this->ClearDisplay();
  // Befehl ausgeben
  $this->WriteLCDData($sendbuf);
 }



 // Cursor ein-/ausschalten ----------------------------------------------------
 public function SendLCDCursor($cstate)
 {
  $sendbuf = $this->SwitchCursor($cstate);
  // Befehl ausgeben
  $this->WriteLCDData($sendbuf);
 }



 // Cursor positionieren -------------------------------------------------------
 public function SendLCDCpos($row, $col)
 {
  if (($row >= $this->rows) || ($col >= $this->cols)) return;

  $sendbuf = $this->CalcCursorPos($row, $col);
  // Befehl ausgeben
  $this->WriteLCDData($sendbuf);
 }



 // Text an bestimmter Cursor-Position ausgeben --------------------------------
 // Zeilenberlauf wird nicht bercksichtigt!
 public function SendLCDText($row, $col, $text)
 {
  if (($row >= $this->rows) || ($col >= $this->cols)) return;
  
  $sendbuf = "";
  // Cursor-Position setzen
  $sendbuf .= $this->CalcCursorPos($row, $col);
  // Text-String erzeugen
  $sendbuf .= $this->MakeTxt($text);
  // Text-String ausgeben
  $this->WriteLCDData($sendbuf);
 }
 
 
 
 // Ausgabe auf das gesamte LCD ------------------------------------------------
 public function SendLCDScreen($text)
 {
  $sendbuf = "";
  $currentline = 0;
  $lines = explode("|", $text);

  // Display lschen
  $sendbuf .= $this->ClearDisplay();

  // Cursor ausschalten
  $sendbuf .= $this->SwitchCursor(false);
  
  foreach($lines as $line)
  {
   // Cursor positionieren
   $sendbuf .= $this->CalcCursorPos($currentline, 0);

   // Text-String erzeugen
   $line = substr($line, 0, $this->cols);
   $sendbuf .=  $this->MakeTxt($line);

   $currentline++;
   if ($currentline >= $this->rows) break;
  }
  // Text-String ausgeben
  $this->WriteLCDData($sendbuf);
 }



 // eigene Zeichen definieren --------------------------------------------------
 public function SendLCDCharDef($num, $data)
 {
  $str = "";
  if ($this->Controller == "ST7036") $str .= chr(0x28);// instruction table 0

  // Speicherposition setzen (0...7)
  $str .= chr((($num & 0x07) << 3) | 0x40);
  $sendbuf = $this->MakeCmd($str);
  // Zeichendaten erzeugen
  $sendbuf .= $this->MakeTxt($data);
  // Command-String ausgeben
  $this->WriteLCDData($sendbuf);
 }



 // Kontrasteinstellung beim ST7036-Controller ---------------------------------
 function SendLCDContrast($ContrastValue)
 {
  // Diese Funktion ist beim Standard-Controller nicht verfgbar!
  if ($this->Controller != "ST7036") return;
    
  $contrast = $ContrastValue & 0x3f;
  $byte1 = 0x29; // instruction table 1
  $byte2 = ($contrast >> 4) | 0x50;
  $byte3 = ($contrast & 0x0f) | 0x70;
  $cstrg = chr($byte1) .chr($byte2) .chr($byte3);
  $sendbuf = $this->MakeCmd($cstrg);
  // Command-String ausgeben
  $this->WriteLCDData($sendbuf);
 }



 // private functions ==========================================================
 // interne Funktionen
 // nicht fr externe Aufrufe verfgbar ========================================
 
 // Cursor-Position berechnen --------------------------------------------------
 private function CalcCursorPos($row, $col)
 {
  $adr = self::$sol[$row] + $col;
  $cpos  = chr($adr | 0x80);
  $sendbuf = $this->MakeNibbles($cpos, true);
  return $sendbuf;
 }

 // Cursor ein-/ausschalten ----------------------------------------------------
 private function SwitchCursor($cstate)
 {
  $cmd = ($cstate) ? 0x0a : 0x80;
  $sendbuf = $this->MakeNibbles($cmd, true);
  return $sendbuf;
 }

 // Display lschen ------------------------------------------------------------
 private function ClearDisplay()
 {
  $sendbuf = $this->MakeNibbles(0x01, true);
  return $sendbuf;
 }
 
 // Command-String erzeugen ----------------------------------------------------
 private function MakeCmd($string)
 {
  return $this->MakeNibbles($string, true);
 }

 // Text-String erzeugen -------------------------------------------------------
 private function MakeTxt($string)
 {
  return $this->MakeNibbles($string, false);
 }

 // Bytes in Nibbles (Halb-Bytes) umwandeln -------------------------------------------------
 // R/S-Bit fr Text setzen, fr Befehle lschen
 private function MakeNibbles($string, $cmd)
 {
  $rs = ($cmd) ? 0 : 0x10;
  $sendbuf = "";
  for ($i=0; $i<strlen($string); $i++)
  {
   $s = ord($string[$i]);
   if (!$cmd && ($s >= 0xc0))
    if ($this->Controller == "ST7036")
     $s = self::$ctab2[chr($s)];
    else
     $s = self::$ctab1[chr($s)];
   $sendbuf .= chr(($s >> 4) | $rs);
   $sendbuf .= chr(($s & 0x0f) | $rs);
  }
  return $sendbuf;
 }
 
 // Ausgabe der Befehls- oder Text-Sequenz -------------------------------------
 private function WriteLCDData($string)
 {
  if ($string != "")
//   TMEX_F29_WriteBytes($this->InstanceID, $string);
   TMEX_F29_WriteBytesMasked($this->InstanceID, $string, 0x1f);
 }
}
?>
